@@ -5,38 +5,48 @@ use 5.010;
 use strict;
 use warnings;
 use namespace::autoclean;
+use TryCatch;
 
-our $VERSION = '1.05'; # VERSION
+our $VERSION = '1.06'; # VERSION
 
 use feature    ();
+use utf8       ();
 use mro        ();
 use IO::File   ();
 use IO::Handle ();
-use Carp       'croak';
+use Carp       qw( croak carp confess cluck );
 
 my %features = (
     10 => [ qw( say state switch ) ],
     12 => ['unicode_strings'],
-    16 => [ qw( unicode_eval evalbytes current_sub array_base fc ) ],
+    16 => [ qw( unicode_eval evalbytes current_sub fc ) ],
     18 => ['lexical_subs'],
-    20 => [ qw( postderef postderef_qq ) ],
+    24 => [ qw( postderef postderef_qq ) ],
+    28 => ['bitwise'],
+);
+
+my %deprecated = (
+    16 => ['array_base'],
 );
 
 my %experiments = (
     20 => ['signatures'],
-    22 => [ qw( refaliasing bitwise ) ],
+    22 => ['refaliasing'],
     26 => ['declared_refs'],
 );
 
 my @function_list = qw(
-    nostrict nowarnings noc3 nobundle noexperiments noskipexperimentalwarnings noautoclean
+    nostrict nowarnings noutf8 noc3 nobundle noexperiments noskipexperimentalwarnings noautoclean nocarp notry
 );
-my @feature_list   = map { @$_ } values %features, values %experiments;
+
+my @feature_list   = map { @$_ } values %features, values %deprecated, values %experiments;
 my ($perl_version) = $^V =~ /^v5\.(\d+)/;
 
+my @autoclean_parameters;
+
 sub import {
     shift;
-    my ( @bundles, @functions, @features );
+    my ( @bundles, @functions, @features, @subclasses );
     for (@_) {
         my $opt = lc $_;
 
@@ -50,16 +60,19 @@ sub import {
             push( @bundles, $1 );
         }
         else {
-            my $v = __PACKAGE__->VERSION;
-            croak( qq{"$opt" is not supported by exact} . ( ($v) ? ' ' . $v : '' ) );
+            push( @subclasses, $opt );
         }
     }
 
     strict->import unless ( grep { $_ eq 'nostrict' } @functions );
     warnings->import unless ( grep { $_ eq 'nowarnings' } @functions );
+
+    unless ( grep { $_ eq 'noutf8' } @functions ) {
+        utf8->import;
+        binmode( $_, ':utf8' ) for ( *STDIN, *STDERR, *STDOUT );
+    }
+
     mro::set_mro( scalar caller(), 'c3' ) unless ( grep { $_ eq 'noc3' } @functions );
-    namespace::autoclean->import( '-cleanee' => scalar caller() )
-        unless ( grep { $_ eq 'noautoclean' } @functions ) ;
 
     if (@bundles) {
         my ($bundle) = sort { $b <=> $a } @bundles;
@@ -69,18 +82,69 @@ sub import {
         feature->import( ':5.' . $perl_version );
     }
 
-    eval {
+    try {
         feature->import($_) for (@features);
         my @experiments = map { @{ $experiments{$_} } } grep { $_ <= $perl_version } keys %experiments;
         feature->import(@experiments) unless ( not @experiments or grep { $_ eq 'noexperiments' } @functions );
-    };
-    if ( my $err = $@ ) {
+    }
+    catch {
+        my $err = @$;
         $err =~ s/\s*at .+? line \d+\.\s+//;
         croak("$err via use of exact");
     }
 
     warnings->unimport('experimental')
         unless ( $perl_version < 18 or grep { $_ eq 'noskipexperimentalwarnings' } @functions );
+
+    my $caller = caller();
+    unless ( grep { $_ eq 'nocarp' } @functions ) {
+        no strict 'refs';
+        *{ $caller . '::croak' }   = \&croak;
+        *{ $caller . '::carp' }    = \&carp;
+        *{ $caller . '::confess' } = \&confess;
+        *{ $caller . '::cluck' }   = \&crcluck;
+    }
+
+    eval qq{
+        package $caller {
+            use TryCatch;
+        };
+    } unless ( grep { $_ eq 'notry' } @functions );
+
+    for my $opt (@subclasses) {
+        my $params = ( $opt =~ s/\(([^\)]+)\)// ) ? [$1] : [];
+
+        ( my $pm = $opt ) =~ s{::|'}{/}g;
+        require "exact/$pm.pm";
+
+        if ( my $e = lcfirst($@) ) {
+            my $v = __PACKAGE__->VERSION;
+            croak(
+                qq{Either "$opt" not supported by exact} .
+                ( ($v) ? ' ' . $v : '' ) .
+                qq{ or $e}
+            );
+        }
+
+        "exact::$opt"->import( scalar caller(), @$params ) if ( "exact::$opt"->can('import') );
+    }
+
+    namespace::autoclean->import( -cleanee => scalar caller(), @autoclean_parameters )
+        unless ( grep { $_ eq 'noautoclean' } @functions ) ;
+}
+
+sub autoclean {
+    my $self = shift;
+
+    my $i = 0;
+    my $caller;
+    while (1) {
+        $caller = caller($i);
+        last if ( not $caller or $caller !~ /^exact\b/ );
+        $i++;
+    }
+
+    @autoclean_parameters = @_;
 }
 
 1;
@@ -97,38 +161,13 @@ exact - Perl pseudo pragma to enable strict, warnings, features, mro, filehandle
 
 =head1 VERSION
 
-version 1.05
+version 1.06
 
 =for markdown [![Build Status](https://travis-ci.org/gryphonshafer/exact.svg)](https://travis-ci.org/gryphonshafer/exact)
 [![Coverage Status](https://coveralls.io/repos/gryphonshafer/exact/badge.png)](https://coveralls.io/r/gryphonshafer/exact)
 
 =head1 SYNOPSIS
 
-Instead of this:
-
-    use strict;
-    use warnings;
-    use feature ':5.23';
-    use feature qw( signatures refaliasing bitwise );
-    use mro 'c3';
-    use IO::File;
-    use IO::Handle;
-    use namespace::autoclean;
-
-    no warnings "experimental::signatures";
-    no warnings "experimental::refaliasing";
-    no warnings "experimental::bitwise";
-
-Type this:
-
-    use exact;
-
-Or for finer control, add some trailing modifiers like a line of the following:
-
-    use exact '5.20';
-    use exact qw( 5.16 nostrict nowarnings noc3 noexperiments noautoclean );
-    use exact qw( noexperiments fc signatures );
-
 =head1 DESCRIPTION
 
 L<exact> is a Perl pseudo pragma to enable strict, warnings, features, mro,
@@ -157,10 +196,53 @@ set C3 style of L<mro>
 
 =item *
 
+use utf8 in the source code context and set STDIN, STROUT, and STRERR to handle UTF8
+
+=item *
+
 enable methods on filehandles
 
+=item *
+
+import L<Carp>'s 4 methods
+
+=item *
+
+import (kinda) L<TryCatch> awesomeness
+
 =back
 
+=for test_synopsis no strict 'subs'
+
+Instead of this:
+
+    use strict;
+    use warnings;
+    use utf8;
+    use open ':std', ':utf8';
+    use feature ':5.23';
+    use feature qw( signatures refaliasing bitwise );
+    use mro 'c3';
+    use IO::File;
+    use IO::Handle;
+    use namespace::autoclean;
+    use Carp qw( croak carp confess cluck );
+    use TryCatch;
+
+    no warnings "experimental::signatures";
+    no warnings "experimental::refaliasing";
+    no warnings "experimental::bitwise";
+
+Type this:
+
+    use exact;
+
+Or for finer control, add some trailing modifiers like a line of the following:
+
+    use exact '5.20';
+    use exact 5.16, nostrict, nowarnings, noc3, noutf8, noexperiments, noautoclean;
+    use exact noexperiments, fc, signatures;
+
 =head1 IMPORT FLAGS
 
 L<exact> supports the following import flags:
@@ -173,6 +255,11 @@ This skips turning on the L<strict> pragma.
 
 This skips turning on the L<warnings> pragma.
 
+=head2 C<noutf8>
+
+This skips turning on UTF8 in the source code context. Also skips setting
+STDIN, STDOUT, and STDERR to expect UTF8.
+
 =head2 C<noc3>
 
 This skips setting C3 L<mro>.
@@ -196,12 +283,21 @@ disabling step.
 
 This skips using L<namespace::autoclean>.
 
-=head2 Explicit Features and Bundles by Name
+=head2 C<nocarp>
+
+This skips importing the 4 L<Carp> methods: C<croak>, C<carp>, C<confess>,
+C<cluck>.
+
+=head2 C<notry>
+
+This skips importing the functionality of L<TryCatch>.
+
+=head1 BUNDLES
 
 You can always provide a list of explicit features and bundles from L<feature>.
 If provided, these will be enabled regardless of the other import flags set.
 
-    use exact qw( noexperiments fc signatures );
+    use exact noexperiments, fc, signatures;
 
 Bundles provided can be exactly like those described in L<feature> or in a
 variety of obvious forms:
@@ -226,6 +322,60 @@ v5.26
 
 =back
 
+=head1 METHODS
+
+=head2 C<autoclean>
+
+Normally, unless you include the C<noautoclean> flag, L<namespace::autoclean>
+will automatically clean your namespace. You can pass flags to autoclean via:
+
+    exact->autoclean( -except => [ qw( method_a method_b) ] );
+
+Note that for this to have any effect, it needs to be called from within your
+module's C<import> method.
+
+=head1 EXTENSIONS
+
+It's possible to write extensions or plugins for L<exact> to provide
+context-specific behavior, provided you are using Perl version 5.14 or newer.
+To activate these extensions, you need to provide their named suffix as a
+parameter to the C<use> of L<exact>.
+
+    # will load "exact" and "exact::class";
+    use exact class;
+
+    # will load "exact" and "exact::role" and turn off UTF8 features;
+    use exact role, noutf8;
+
+It's possible to provide parameters to the C<import> method of the extension.
+
+    # will load "exact" and "exact::answer" and pass "42" to the import method
+    use exact 'answer(42)';
+
+=head2 Writing Extensions
+
+An extension may but is not required to have an C<import> method. If such a
+method does exist, it will be passed: the package name, the name of the caller
+of L<exact>, and any parameters passed.
+
+    package exact::example;
+    use exact;
+
+    sub import {
+        my ( $self, $caller, $params ) = @_;
+        {
+            no strict 'refs';
+            *{ $caller . '::example' } = \&example;
+        }
+        exact->autoclean( -except => ['example'] );
+    }
+
+    sub example {
+        say 42;
+    }
+
+    1;
+
 =head1 SEE ALSO
 
 You can look for additional information at:
